@@ -2,8 +2,6 @@ from .. import util
 
 machinery = util.import_importlib('importlib.machinery')
 
-import collections
-import sys
 import unittest
 
 
